/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <iostream>
#include <unistd.h>
#include "baiduspeechengine.h"

std::vector<char> readAudioData(const std::string &filePath) {
    std::ifstream file(filePath, std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Failed to open file: " << filePath << std::endl;
        return {};
    }

    file.seekg(0, std::ios::end);
    std::streampos fileSize = file.tellg();
    file.seekg(0, std::ios::beg);
    std::vector<char> audioData(fileSize);
    file.read(audioData.data(), fileSize);
    return audioData;
}

std::vector<std::vector<char>>
splitAudioData(const std::vector<char> &audioData, size_t chunkSize) {
    std::vector<std::vector<char>> chunks;
    size_t dataSize = audioData.size();
    size_t numChunks = dataSize / chunkSize;
    size_t remainder = dataSize % chunkSize;

    for (size_t i = 0; i < numChunks; ++i) {
        chunks.push_back(
            std::vector<char>(audioData.begin() + i * chunkSize,
                              audioData.begin() + (i + 1) * chunkSize));
    }

    if (remainder != 0) {
        chunks.push_back(
            std::vector<char>(audioData.end() - remainder, audioData.end()));
    }

    return chunks;
}

void testSpeech() {
    BaiduSpeechEngine *engine = new BaiduSpeechEngine();

    engine->setRecognizingCallback(
        [](ai_engine::lm::speech::RecognitionResult result) {
            std::cout << "MID_TEXT: " << result.text << std::endl;
        });
    engine->setRecognizedCallback(
        [](ai_engine::lm::speech::RecognitionResult result) {
            std::cout << "FIN_TEXT: " << result.text << std::endl;
        });

    engine->setSynthesizingCallback(
        [](ai_engine::lm::speech::SynthesisResult result) {
            std::cout << result.audioData.size() << std::endl;
        });
    engine->setSynthesizedCallback([](ai_engine::lm::speech::ResultType type) {
        std::cout << "Synthesize finished..." << std::endl;
    });

    engine->setAppId("fakeAppId");
    engine->setApiKey("fakeApiKey");
    engine->setSecretKey("fakeSecretKey");

    auto audioData = readAudioData("16k_10.pcm");
    ai_engine::ErrorCode ret;
    ret = engine->recognizeOnce("\{\"rate\": 1600}", audioData);
    std::cout << "recongnize once: " << (int)ret << std::endl;
    assert(ret == ai_engine::ErrorCode::Success);

    ret = engine->startContinuousRecognition("\{\"rate\": 1600}");
    assert(ret == ai_engine::ErrorCode::Success);
    auto audioDatas = splitAudioData(audioData, 32000);
    for (auto &data : audioDatas) {
        ret = engine->writeContinuousRecognitionAudioData(data);
        assert(ret == ai_engine::ErrorCode::Success);
        sleep(1);
    }
    sleep(5);
    engine->stopContinuousRecognition();

    std::string text = "麒麟软件，风行天下";
    ret = engine->synthesizeOnce("\{\"speed\": 5}", text);
    std::cout << "synthesize once: " << (int)ret << std::endl;
    assert(ret == ai_engine::ErrorCode::Success);
}

int main(int argc, char *argv[]) {
    testSpeech();
    return 0;
}